about summary refs log tree commit diff
path: root/library/std/src/sys_common
diff options
context:
space:
mode:
authorMara Bos <m-ou.se@m-ou.se>2022-06-16 11:23:31 +0200
committerMara Bos <m-ou.se@m-ou.se>2022-06-16 12:09:12 +0200
commitd72294491c5d28449c49b884b620cc24b3cf010f (patch)
treea9df2c0fc5270ca3d1188bf63de53246ebad25e6 /library/std/src/sys_common
parent392d2728683f140f6125732240e462c43c5caff4 (diff)
downloadrust-d72294491c5d28449c49b884b620cc24b3cf010f.tar.gz
rust-d72294491c5d28449c49b884b620cc24b3cf010f.zip
Leak pthreax_mutex_t when it's dropped while locked.
Diffstat (limited to 'library/std/src/sys_common')
-rw-r--r--library/std/src/sys_common/lazy_box.rs19
1 files changed, 16 insertions, 3 deletions
diff --git a/library/std/src/sys_common/lazy_box.rs b/library/std/src/sys_common/lazy_box.rs
index 647c13d2437..63c3316bdeb 100644
--- a/library/std/src/sys_common/lazy_box.rs
+++ b/library/std/src/sys_common/lazy_box.rs
@@ -21,8 +21,21 @@ pub(crate) trait LazyInit {
     ///
     /// It might be called more than once per LazyBox, as multiple threads
     /// might race to initialize it concurrently, each constructing and initializing
     /// their own box. (All but one of them will be destroyed right after.)
     fn init() -> Box<Self>;
+
+    /// Any surplus boxes from `init()` that lost the initialization race
+    /// are passed to this function for disposal.
+    ///
+    /// The default implementation calls destroy().
+    fn cancel_init(x: Box<Self>) {
+        Self::destroy(x);
+    }
+
+    /// This is called to destroy a used box.
+    ///
+    /// The default implementation just drops it.
+    fn destroy(_: Box<Self>) {}
 }
 
 impl<T: LazyInit> LazyBox<T> {
@@ -45,7 +58,7 @@ impl<T: LazyInit> LazyBox<T> {
             Err(ptr) => {
                 // Lost the race to another thread.
                 // Drop the box we created, and use the one from the other thread instead.
-                drop(unsafe { Box::from_raw(new_ptr) });
+                T::cancel_init(unsafe { Box::from_raw(new_ptr) });
                 ptr
             }
         }
@@ -71,7 +84,7 @@ impl<T: LazyInit> Drop for LazyBox<T> {
     fn drop(&mut self) {
         let ptr = *self.ptr.get_mut();
         if !ptr.is_null() {
-            drop(unsafe { Box::from_raw(ptr) });
+            T::destroy(unsafe { Box::from_raw(ptr) });
         }
     }
 }
@@ -21,8 +21,21 @@ pub(crate) trait LazyInit {
     /// It might be called more than once per LazyBox, as multiple threads
     /// might race to initialize it concurrently, each constructing and initializing
-    /// their own box. (All but one of them will be destroyed right after.)
+    /// their own box. All but one of them will be passed to `cancel_init` right after.
     fn init() -> Box<Self>;